<?php

namespace Litecms\Subcategory\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Subcategory\Http\Requests\SubcategoriesRequest;
use Litecms\Subcategory\Interfaces\SubcategoriesRepositoryInterface;
use Litecms\Subcategory\Models\Subcategories;

/**
 * Resource controller class for subcategories.
 */
class SubcategoriesResourceController extends BaseController
{

    /**
     * Initialize subcategories resource controller.
     *
     * @param type SubcategoriesRepositoryInterface $subcategories
     *
     * @return null
     */
    public function __construct(SubcategoriesRepositoryInterface $subcategories)
    {
        parent::__construct();
        $this->repository = $subcategories;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Subcategory\Repositories\Criteria\SubcategoriesResourceCriteria::class);
    }

    /**
     * Display a list of subcategories.
     *
     * @return Response
     */
    public function index(SubcategoriesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Subcategory\Repositories\Presenter\SubcategoriesPresenter::class)
                ->$function();
        }

        $subcategories = $this->repository->paginate();

        return $this->response->title(trans('subcategory::subcategories.names'))
            ->view('subcategory::subcategories.index', true)
            ->data(compact('subcategories'))
            ->output();
    }

    /**
     * Display subcategories.
     *
     * @param Request $request
     * @param Model   $subcategories
     *
     * @return Response
     */
    public function show(SubcategoriesRequest $request, Subcategories $subcategories)
    {

        if ($subcategories->exists) {
            $view = 'subcategory::subcategories.show';
        } else {
            $view = 'subcategory::subcategories.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('subcategory::subcategories.name'))
            ->data(compact('subcategories'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new subcategories.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SubcategoriesRequest $request)
    {

        $subcategories = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('subcategory::subcategories.name')) 
            ->view('subcategory::subcategories.create', true) 
            ->data(compact('subcategories'))
            ->output();
    }

    /**
     * Create new subcategories.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SubcategoriesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $subcategories                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('subcategory::subcategories.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('subcategory/subcategories/' . $subcategories->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/subcategory/subcategories'))
                ->redirect();
        }

    }

    /**
     * Show subcategories for editing.
     *
     * @param Request $request
     * @param Model   $subcategories
     *
     * @return Response
     */
    public function edit(SubcategoriesRequest $request, Subcategories $subcategories)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('subcategory::subcategories.name'))
            ->view('subcategory::subcategories.edit', true)
            ->data(compact('subcategories'))
            ->output();
    }

    /**
     * Update the subcategories.
     *
     * @param Request $request
     * @param Model   $subcategories
     *
     * @return Response
     */
    public function update(SubcategoriesRequest $request, Subcategories $subcategories)
    {
        try {
            $attributes = $request->all();

            $subcategories->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('subcategory::subcategories.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('subcategory/subcategories/' . $subcategories->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('subcategory/subcategories/' . $subcategories->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the subcategories.
     *
     * @param Model   $subcategories
     *
     * @return Response
     */
    public function destroy(SubcategoriesRequest $request, Subcategories $subcategories)
    {
        try {

            $subcategories->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('subcategory::subcategories.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('subcategory/subcategories/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('subcategory/subcategories/' . $subcategories->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple subcategories.
     *
     * @param Model   $subcategories
     *
     * @return Response
     */
    public function delete(SubcategoriesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('subcategory::subcategories.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('subcategory/subcategories'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/subcategory/subcategories'))
                ->redirect();
        }

    }

    /**
     * Restore deleted subcategories.
     *
     * @param Model   $subcategories
     *
     * @return Response
     */
    public function restore(SubcategoriesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('subcategory::subcategories.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/subcategory/subcategories'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/subcategory/subcategories/'))
                ->redirect();
        }

    }

}
